@@ -194,7 +194,7 @@ menu_add_tog( GtkWidget *menu, const char *name, GtkSignalFunc cb, void *user )
 {
 	GtkWidget *tog;
 
-	tog = gtk_check_menu_item_new_with_label( name );
+	tog = gtk_check_menu_item_new_with_mnemonic( name );
 	gtk_menu_shell_append( GTK_MENU_SHELL( menu ), tog );
 	gtk_widget_show( tog );
 	gtk_signal_connect( GTK_OBJECT( tog ), "toggled", cb, user );
@@ -470,16 +470,16 @@ get_geditable_name( GtkWidget *text, char *out, int sz )
 	name = gtk_editable_get_chars( GTK_EDITABLE( text ), 0, -1 );
 	tname = trim_nonalpha( name );
 	if( !tname ) {
-		FREEF( g_free, name );
+		IM_FREEF( g_free, name );
 		error_top( _( "Bad identifier." ) );
 		error_top( 
-			_( "Enter an identifier. Identifiers start with\n"
-			"a letter, and then contain only letters, numbers,\n"
+			_( "Enter an identifier. Identifiers start with "
+			"a letter, and then contain only letters, numbers, "
 			"apostrophy and underscore." ) );
 		return( FALSE );
 	}
 	im_strncpy( out, tname, sz );
-	FREEF( g_free, name );
+	IM_FREEF( g_free, name );
 
 	return( TRUE );
 }
@@ -491,7 +491,7 @@ get_geditable_string( GtkWidget *text, char *out, int sz )
 
 	str = gtk_editable_get_chars( GTK_EDITABLE( text ), 0, -1 );
 	im_strncpy( out, str, sz );
-	FREEF( g_free, str );
+	IM_FREEF( g_free, str );
 
 	return( TRUE );
 }
@@ -505,11 +505,11 @@ get_geditable_filename( GtkWidget *text, char *out, int sz )
 	filename = gtk_editable_get_chars( GTK_EDITABLE( text ), 0, -1 );
 	tfilename = trim_white( filename );
 	if( !is_valid_filename( tfilename ) ) {
-		FREEF( g_free, filename );
+		IM_FREEF( g_free, filename );
 		return( FALSE );
 	}
 	im_strncpy( out, tfilename, sz );
-	FREEF( g_free, filename );
+	IM_FREEF( g_free, filename );
 
 	return( TRUE );
 }
@@ -527,18 +527,18 @@ get_geditable_double( GtkWidget *text, double *out )
 	if( end == txt ) {
 		error_top( _( "Bad floating point number." ) );
 		error_sub( _( "\"%s\" is not a floating point number." ), txt );
-		FREEF( g_free, txt );
+		IM_FREEF( g_free, txt );
 		return( FALSE );
 	}
 
 	if( strspn( end, WHITESPACE ) != strlen( end ) ) {
 		error_top( _( "Bad floating point number." ) );
 		error_sub( _( "Extra characters \"%s\" after number." ), end );
-		FREEF( g_free, txt );
+		IM_FREEF( g_free, txt );
 		return( FALSE );
 	}
 
-	FREEF( g_free, txt );
+	IM_FREEF( g_free, txt );
 	*out = t;
 
 	return( TRUE );
@@ -557,10 +557,10 @@ get_geditable_int( GtkWidget *text, int *n )
 	if( sscanf( txt, "%i", &i ) != 1 ) {
 		error_top( _( "Bad integer." ) );
 		error_sub( _( "\"%s\" is not an integer." ), txt );
-		FREEF( g_free, txt );
+		IM_FREEF( g_free, txt );
 		return( FALSE );
 	}
-	FREEF( g_free, txt );
+	IM_FREEF( g_free, txt );
 	*n = i;
 
 	return( TRUE );
@@ -913,8 +913,7 @@ void *
 gobject_print( GObject *gobject )
 {
 	printf( "%s (0x%x)\n", 
-		G_OBJECT_TYPE_NAME( gobject ),
-		(unsigned int) gobject ); 
+		G_OBJECT_TYPE_NAME( gobject ), GPOINTER_TO_UINT( gobject ) );
 
 	return( NULL );
 }
