@@ -36,6 +36,10 @@
 #define DEBUG_HEAP_GC
  */
 
+/* Trace create/destroy of static strings.
+#define DEBUG_STATIC
+ */
+
 #include "ip.h"
 
 static iObjectClass *parent_class = NULL;
@@ -202,8 +206,8 @@ heapblock_free( HeapBlock *hb )
 	if( hb->next )
 		heapblock_free( hb->next );
 	if( hb->node )
-		FREE( hb->node );
-	FREE( hb );
+		IM_FREE( hb->node );
+	IM_FREE( hb );
 }
 
 static void
@@ -219,20 +223,46 @@ heap_dispose( GObject *gobject )
 	 */
 	slist_map( heap->itable, (SListMapFn) imageinfo_dump, NULL );
 
-	FREEFI( g_source_remove, heap->gc_tid );
+	IM_FREEF( g_source_remove, heap->gc_tid );
 
 	G_OBJECT_CLASS( parent_class )->dispose( gobject );
 }
 
+static void
+heap_static_string_free( HeapStaticString *string )
+{
+	assert( string->count == 0 );
+
+	IM_FREE( string->text );
+	heap_unregister_element( string->heap, &string->e );
+	IM_FREE( string );
+}
+
+static gboolean 
+heap_finalize_static_check( const char *text, HeapStaticString *string )
+{
+	assert( string->count == 0 );
+
+	heap_static_string_free( string );
+
+	return( TRUE );
+}
+
 static void
 heap_finalize( GObject *gobject )
 {
 	Heap *heap = HEAP( gobject );
 
+	if( heap->statics )
+		g_hash_table_foreach_remove( heap->statics, 
+			(GHRFunc) heap_finalize_static_check, NULL );
+	IM_FREEF( g_hash_table_destroy, heap->statics );
+
 	if( heap->hb )
 		heapblock_free( heap->hb );
-	FREEF( g_hash_table_destroy, heap->emark );
-	FREEF( g_slist_free, heap->rmark );
+	IM_FREEF( g_hash_table_destroy, heap->emark );
+
+	IM_FREEF( g_slist_free, heap->rmark );
 
 	heap_all = g_slist_remove( heap_all, heap );
 
@@ -249,7 +279,7 @@ heap_info( iObject *iobject, BufInfo *buf )
 		if( heap->compile->sym ) {
 			symbol_qualified_name( heap->compile->sym, buf );
 			buf_appendf( buf, "(0x%x) (sym)\n", 
-				(unsigned int) heap->compile->sym ); 
+				GPOINTER_TO_UINT( heap->compile->sym ) ); 
 		}
 		else
 			buf_appendf( buf, "(compile, but no sym)\n" );
@@ -351,19 +381,24 @@ static void
 heap_init( Heap *heap )
 {
 	heap->compile = NULL;
+
 	heap->max_fn = NULL;
 	heap->mxb = -1;
 	heap->rsz = 0;
 	heap->nb = 0;
 	heap->hb = NULL;
 	heap->free = NULL;
-	heap->emark = g_hash_table_new( NULL, g_direct_equal );
-	heap->rmark = NULL;
-	heap->nfree = 0;
+
 	heap->ncells = 0;
-	heap->itable = NULL;
+	heap->nfree = 0;
 	heap->serial = 0;
 	heap->filled = FALSE;
+
+	heap->emark = g_hash_table_new( NULL, g_direct_equal );
+	heap->rmark = NULL;
+	heap->itable = NULL;
+	heap->statics = g_hash_table_new( g_str_hash, g_str_equal );
+
 	heap->gc_tid = 0;
 
 	heap_all = g_slist_prepend( heap_all, heap );
@@ -486,10 +521,6 @@ heap_mark_image( Imageinfo *ii, Heap *heap )
 	imageinfo_mark( ii );
 }
 
-/* Need fwd ref.
- */
-static void *mark_pelement( PElement *root, Heap *heap );
-
 /* Mark a tree. Avoid recursion because of the danger of C stack overflow on 
  * large heaps.
  */
@@ -633,6 +664,24 @@ free_images( Imageinfo *ii )
 	return( NULL );
 }
 
+/* Check for unreffed static strings.
+ */
+static gboolean 
+heap_static_check( const char *text, HeapStaticString *string )
+{
+	if( string->count == 0 ) {
+#ifdef DEBUG_STATIC
+		printf( "heap_static_check: destroying \"%s\"\n", text );
+#endif /*DEBUG_STATIC*/
+
+		heap_static_string_free( string );
+
+		return( TRUE );
+	}
+	else
+		return( FALSE );
+}
+
 /* Do a garbage collect.
  */
 gboolean
@@ -659,6 +708,11 @@ heap_gc( Heap *heap )
 	if( slist_map( heap->itable, (SListMapFn) heap_image_clear, NULL ) )
 		return( FALSE );
 
+	/* Free all unreffed static strings.
+	 */
+	g_hash_table_foreach_remove( heap->statics, 
+		(GHRFunc) heap_static_check, NULL );
+
 	/* All flgs should be clear, so just mark.
 	 */
 	g_hash_table_foreach( heap->emark, (GHFunc) mark_element, heap );
@@ -669,14 +723,14 @@ heap_gc( Heap *heap )
 	heap->free = NULL;
 	ncells = nfree = nblocks = 0;
 	for( hb = heap->hb; hb; hb = hb->next ) {
+		const int sz = hb->sz;
 		int i;
 
-		for( i = 0; i < hb->sz; i++ ) {
-			HeapNode *hn = &hb->node[i];
+		for( i = 0; i < sz; i++ ) {
+			HeapNode * const hn = &hb->node[i];
 
 			if( !(hn->flgs & FLAG_MARK) ) {
 				hn->type = TAG_FREE;
-				hn->flgs = 0;
 				PPUTLEFT( hn, ELEMENT_NODE, heap->free );
 #ifdef DEBUG_HEAP_GC
 				/* Not necessary, but may be helpful to zap
@@ -685,7 +739,7 @@ heap_gc( Heap *heap )
 				PPUTRIGHT( hn, ELEMENT_NODE, NULL );
 #endif /*DEBUG_HEAP_GC*/
 				heap->free = hn;
-				nfree++;
+				nfree += 1;
 			}
 
 			hn->flgs &= FLAG_MARK ^ FLAG_ALL;
@@ -730,7 +784,7 @@ heap_gc_request_cb( Heap *heap )
 void
 heap_gc_request( Heap *heap )
 {
-	FREEFI( g_source_remove, heap->gc_tid );
+	IM_FREEF( g_source_remove, heap->gc_tid );
 	heap->gc_tid = g_timeout_add( 1000, 
 		(GSourceFunc) heap_gc_request_cb, heap );
 }
@@ -827,12 +881,12 @@ heap_getmem( Heap *heap )
 
 			buf_init_static( &buf, txt, 100 );
 			compile_name( heap->compile, &buf );
-			error_sub( _( "The compile heap for %s has filled.\n"
+			error_sub( _( "The compile heap for %s has filled. "
 				"Make it smaller and less complicated." ),
 				buf_all( &buf ) );
 		}
 		else
-			error_sub( _( "The main calculation heap has filled.\n"
+			error_sub( _( "The main calculation heap has filled. "
 				"Raise the heap size limit in Preferences." ) );
 		heap->filled = TRUE;
 		return( NULL );
@@ -1505,7 +1559,7 @@ copy_node( Heap *heap, HeapNode *ri[], HeapNode *hn, PElement *out )
 	if( hn->type == TAG_SHARED ) {
 		/* RHS of SHARE is the index of this share node.
 		 */
-		i = (int) GETRIGHT( hn );
+		i = GPOINTER_TO_INT( GETRIGHT( hn ) );
 
 		/* Skip to shared section.
 		 */
@@ -1526,7 +1580,7 @@ copy_node( Heap *heap, HeapNode *ri[], HeapNode *hn, PElement *out )
 		/* Must have already copied this SHARE, just link back.
 		 */
 		hn1 = GETLEFT( hn );
-		i = (int) GETRIGHT( hn1 );
+		i = GPOINTER_TO_INT( GETRIGHT( hn1 ) );
 		PEPUTP( out, ELEMENT_NODE, ri[i] );
 
 		/* Done!
@@ -1624,6 +1678,7 @@ heap_copy( Heap *heap, Compile *compile, PElement *out )
 	case ELEMENT_TAG:
 	case ELEMENT_ELIST:
 	case ELEMENT_IMAGE:
+	case ELEMENT_STATIC:
 		/* Copy value.
 		 */
 		PEPUTP( out, root->type, root->ele );
@@ -1640,3 +1695,57 @@ heap_copy( Heap *heap, Compile *compile, PElement *out )
 
 	return( TRUE );
 }
+
+HeapStaticString *
+heap_static_string_new( Heap *heap, const char *text )
+{
+	HeapStaticString *string;
+
+	if( !(string = g_hash_table_lookup( heap->statics, text )) ) {
+		PElement pe;
+
+		if( !(string = IM_NEW( NULL, HeapStaticString )) )
+			return( NULL );
+
+		string->heap = heap;
+		string->text = NULL;
+		string->count = 0;
+		string->e.type = ELEMENT_NOVAL;
+		string->e.ele = NULL;
+		heap_register_element( heap, &string->e );
+
+		PEPOINTE( &pe, &string->e );
+		if( !(string->text = im_strdup( NULL, text )) ||
+			!heap_string_new( heap, text, &pe ) ) {
+			heap_static_string_free( string );
+			return( NULL );
+		}
+
+#ifdef DEBUG_STATIC
+		printf( "heap_static_string_new: \"%s\"\n", text );
+#endif /*DEBUG_STATIC*/
+
+		g_hash_table_insert( heap->statics, string->text, string );
+	}
+
+	assert( string->count >= 0 );
+	string->count += 1;
+
+	return( string );
+}
+
+void *
+heap_static_string_destroy( HeapStaticString *string )
+{
+	assert( g_hash_table_lookup( string->heap->statics, string->text ) ==
+		string );
+	assert( string->count >= 0 );
+
+	string->count -= 1;
+
+	/* Don't destroy on count == 0; GC frees statics for us next time it
+	 * runs.
+	 */
+
+	return( NULL );
+}
