@@ -564,7 +564,7 @@ static GtkToggleActionEntry imageview_toggle_actions[] = {
 		G_CALLBACK( imageview_show_status_action_cb ), TRUE },
 
 	{ "Control",
-		NULL, N_( "_Display control" ), NULL,
+		NULL, N_( "_Display Control" ), NULL,
 		N_( "Show display control bar" ),
 		G_CALLBACK( imageview_show_convert_action_cb ), TRUE },
 
